package com.fw.cos.fw_cos;

import android.app.Activity;
import android.util.Log;
import java.util.HashMap;

import androidx.annotation.NonNull;
import android.content.Context;
import android.app.Application;

import io.flutter.embedding.engine.plugins.FlutterPlugin;
import io.flutter.embedding.engine.plugins.activity.ActivityPluginBinding;
import io.flutter.embedding.engine.plugins.activity.ActivityAware;
import io.flutter.plugin.common.MethodCall;
import io.flutter.plugin.common.MethodChannel;
import io.flutter.plugin.common.MethodChannel.MethodCallHandler;
import io.flutter.plugin.common.MethodChannel.Result;

import com.tencent.cos.xml.CosXmlService;
import com.tencent.cos.xml.CosXmlServiceConfig;
import com.tencent.cos.xml.exception.CosXmlClientException;
import com.tencent.cos.xml.exception.CosXmlServiceException;
import com.tencent.cos.xml.listener.CosXmlProgressListener;
import com.tencent.cos.xml.listener.CosXmlResultListener;
import com.tencent.cos.xml.model.CosXmlRequest;
import com.tencent.cos.xml.model.CosXmlResult;
import com.tencent.cos.xml.transfer.COSXMLUploadTask;
import com.tencent.cos.xml.transfer.TransferConfig;
import com.tencent.cos.xml.transfer.TransferManager;

/** FwCosPlugin */
public class FwCosPlugin implements FlutterPlugin, MethodCallHandler, ActivityAware {
  /// The MethodChannel that will the communication between Flutter and native Android
  ///
  /// This local reference serves to register the plugin with the Flutter Engine and unregister it
  /// when the Flutter Engine is detached from the Activity
  private MethodChannel channel;
  private Context mContext;
  private Activity mActivity;
  // Registrar registrar;

  // This static function is optional and equivalent to onAttachedToEngine. It supports the old
  // pre-Flutter-1.12 Android projects. You are encouraged to continue supporting
  // plugin registration via this function while apps migrate to use the new Android APIs
  // post-flutter-1.12 via https://flutter.dev/go/android-project-migration.
  //
  // It is encouraged to share logic between onAttachedToEngine and registerWith to keep
  // them functionally equivalent. Only one of onAttachedToEngine or registerWith will be called
  // depending on the user's project. onAttachedToEngine or registerWith must both be defined
  // in the same class.
  //  public static void registerWith(Registrar registrar) {
  //    final MethodChannel channel = new MethodChannel(registrar.messenger(), "fw_cos");
  //    channel.setMethodCallHandler(new FwCosPlugin(registrar, channel));
  //  }
  // flutter 1.12之前的注册方法，没测试，估计用不了
  //  public FwCosPlugin(PluginRegistry.Registrar registrar, MethodChannel channel) {
  //    this.registrar = registrar;
  //    this.channel = channel;
  //  }

  @Override
  public void onAttachedToEngine(@NonNull FlutterPluginBinding flutterPluginBinding) {
    channel = new MethodChannel(flutterPluginBinding.getFlutterEngine().getDartExecutor(), "fw_cos");
    mContext = flutterPluginBinding.getApplicationContext();
    channel.setMethodCallHandler(this);
  }

  @Override
  public void onMethodCall(@NonNull MethodCall call, @NonNull final Result result) {
    if (call.method.equals("getPlatformVersion")) {
      result.success("Android " + android.os.Build.VERSION.RELEASE);
    } else if(call.method.equals("getNative")) {
      result.success("getNative");
    } else if(call.method.equals("uploadFile")){
      Log.d("onMethodCall", "uploadFile");

      String secretId = call.argument("secretId"); //secretId
      String secretKey = call.argument("secretKey"); //secretKey

      // 此秘钥计算方法与项目中用到的不符合，所以不使用该方法生成秘钥
      // QCloudCredentialProvider myCredentialProvider =
      //        new ShortTimeCredentialProvider(secretId, secretKey, 300);
      LocalSessionCredentialProvider myCredentialProvider = new LocalSessionCredentialProvider(call.<String>argument("secretId"), call.<String>argument("secretKey"), call.<String>argument("sessionToken"), Long.parseLong(call.argument("expiredTime").toString()));

      String region = call.argument("region"); // region
      String bucket = call.argument("bucket"); // bucket
      final String localPath = call.argument("localPath"); // localPath
      final String cosPath = call.argument("cosPath"); // cosPath

      /// 初始化 COS Service
      // 创建 CosXmlServiceConfig 对象，根据需要修改默认的配置参数
      CosXmlServiceConfig serviceConfig = new CosXmlServiceConfig.Builder()
              .setRegion(region)
              .isHttps(true) // 使用 HTTPS 请求, 默认为 HTTP 请求
              .builder();

      CosXmlService cosXmlService = new CosXmlService(mContext, serviceConfig, myCredentialProvider);

      // 初始化 TransferConfig，这里使用默认配置，如果需要定制，请参考 SDK 接口文档
      TransferConfig transferConfig = new TransferConfig.Builder().build();
      //初始化 TransferManager
      TransferManager transferManager = new TransferManager(cosXmlService, transferConfig);
      //上传文件
      COSXMLUploadTask cosxmlUploadTask = transferManager.upload(bucket, cosPath, localPath, null);

      final HashMap<String, Object> data = new HashMap<>();
      data.put("localPath", localPath);
      data.put("cosPath", cosPath);

      Log.d("onMethodCall", "startUpload");

      cosxmlUploadTask.setCosXmlProgressListener(new CosXmlProgressListener() {
        @Override
        public void onProgress(final long complete, final long target) {
          Log.d("onProgress", complete + " : " + target);
          mActivity.runOnUiThread(new Runnable() {
            @Override
            public void run() {
              HashMap<String, Object> progress = new HashMap<>();
              progress.put("cosPath", cosPath);
              progress.put("localPath", localPath);
              progress.put("progress", complete * 100.0 / target);
              channel.invokeMethod("onProgress", progress);
            }
          });
        }
      });

      //设置返回结果回调
      cosxmlUploadTask.setCosXmlResultListener(new CosXmlResultListener() {
        @Override
        public void onSuccess(CosXmlRequest request, CosXmlResult httpResult) {
          Log.d("onSuccess", httpResult.printResult());
          mActivity.runOnUiThread(new Runnable() {
            @Override
            public void run() {
              result.success(data);
              channel.invokeMethod("onSuccess", cosPath);
            }
          });
        }

        @Override
        public void onFail(CosXmlRequest request, final CosXmlClientException exception, CosXmlServiceException serviceException) {
          Log.d("onFail", exception.toString() + serviceException.toString());
          data.put("message", (exception.toString() + serviceException.toString()));
          mActivity.runOnUiThread(new Runnable() {
            @Override
            public void run() {
              result.error("400", "error", exception.toString());
              channel.invokeMethod("onFailed", data);
            }
          });

          if (exception != null) {
            exception.printStackTrace();
          } else {
            serviceException.printStackTrace();
          }
        }
      });
    } else {
      result.notImplemented();
    }
  }

  @Override
  public void onDetachedFromEngine(@NonNull FlutterPluginBinding binding) {
    channel.setMethodCallHandler(null);
    channel = null;
  }

  ///activity 生命周期
  @Override
  public void onAttachedToActivity(@NonNull ActivityPluginBinding binding) {
    mActivity = binding.getActivity();
  }

  @Override
  public void onDetachedFromActivity() {
    mActivity = null;
  }

  @Override
  public void onDetachedFromActivityForConfigChanges() {
  }

  @Override
  public void onReattachedToActivityForConfigChanges(@NonNull ActivityPluginBinding binding) {

  }
}
